! Copyright (C) 2019-2021:
!   Max Planck Institut fuer extraterrestrische Physik, Leibniz Institute for Astrophysics Potsdam (AIP)
!
! This file is part of eRosita's ESASS (https://erosita.mpe.mpg.de/). It is free software; you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License as published by the Free Software Foundation; either
! version 2 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
! USA, or see https://www.gnu.org/licenses/.
!

module CFITSIO_IFACES 

!------------------------------------------------------------------------------- 
! Provides Fortran90 interface blocks to selected CFITSIO routines 
! Author: GLamer, AIP         date: 24-May-2010
! update  GL:                 date: 24-Jun-2010 fixed 'use numkind' statements
! update  GL:                 date: 06-Aug-2010 changed to F77 style explicit shape 
!                                               arrays (see NOTE).
!   
! NOTE: Array declarations must follow   F77 style (explicit shape).  
!       F90 style assumed shape declarations (dimension(:)) will lead to 
!       segmention faults during runtime.                                             
!------------------------------------------------------------------------------- 



  interface  FTGIOU
    subroutine  FTGIOU(unit,status)
      use numkind
      integer(kind=int32) :: unit,status
    end subroutine FTGIOU 
  end interface  FTGIOU

  interface FTOPEN
    subroutine FTOPEN(unit,filename,readwrite,blocksize,status)
      use numkind
      integer(kind=int32) :: unit,readwrite,blocksize,status
      character(len=*):: filename
    end subroutine FTOPEN
  end interface FTOPEN

  interface FTCLOS
    subroutine FTCLOS(unit,status)
      use numkind
      integer(kind=int32) :: unit,status
    end subroutine FTCLOS
  end interface FTCLOS

  interface  FTMNHD 
    subroutine FTMNHD(unit,hdutype,extname,extver,status)
      use numkind
      integer(kind=int32) :: unit,hdutype,extver,status
      character(len=*):: extname
    end subroutine  FTMNHD
  end interface  FTMNHD


  interface  FTGCNO
    subroutine FTGCNO(unit,casesens, colname, col, status)
      use numkind
      integer(kind=int32) :: unit,col,status
      logical(kind=log32) :: casesens
      character(len=*):: colname
    end subroutine  FTGCNO
  end interface  FTGCNO




  ! FTGCVx:

  
  interface FTGCVB
    subroutine  FTGCVB(unit,colnum,frow,felem,nelements,nullval,values,anyf,status)
      use numkind
      integer(kind=int32) :: unit,colnum,frow,felem,nelements,status
      logical(kind=log32) :: anyf
      integer(kind=int8) :: nullval
      integer(kind=int8), dimension(nelements) :: values  
    end subroutine  FTGCVB
  end interface FTGCVB
 
  
  interface FTGCVI
    subroutine  FTGCVI(unit,colnum,frow,felem,nelements,nullval,values,anyf,status)
      use numkind
      integer(kind=int32) :: unit,colnum,frow,felem,nelements,status
      logical(kind=log32) :: anyf
      integer(kind=int16) :: nullval
      integer(kind=int16), dimension(nelements) :: values  
    end subroutine  FTGCVI
  end interface FTGCVI
 
  interface FTGCVJ
    subroutine  FTGCVJ(unit,colnum,frow,felem,nelements,nullval,values,anyf,status)
      use numkind
      integer(kind=int32) :: unit,colnum,frow,felem,nelements,status
      logical(kind=log32) :: anyf
      integer(kind=int32) :: nullval
      integer(kind=int32), dimension(nelements) :: values  
    end subroutine  FTGCVJ
  end interface FTGCVJ



 
  interface FTGCVE
    subroutine  FTGCVE(unit,colnum,frow,felem,nelements,nullval,values,anyf,status)
      use numkind
      integer(kind=int32) :: unit,colnum,frow,felem,nelements,status
      logical(kind=log32) :: anyf
      real(kind=real32) :: nullval
      real(kind=real32), dimension(nelements) :: values  
    end subroutine  FTGCVE
  end interface FTGCVE


! test remove 2/2/2013
  interface FTGCVD
    subroutine  FTGCVD(unit,colnum,frow,felem,nelements,nullval,values,anyf,status)
      use numkind
      integer(kind=int32) :: unit,colnum,frow,felem,nelements,status
      logical(kind=log32) :: anyf
      real(kind=real64) :: nullval
      real(kind=real64), dimension(nelements) :: values  
    end subroutine  FTGCVD
  end interface FTGCVD

  interface FTGCVS
    subroutine  FTGCVS(unit,colnum,frow,felem,nelements,nullval,values,anyf,status)
      use numkind
      integer(kind=int32) :: unit,colnum,frow,felem,nelements,status
      logical(kind=log32) :: anyf
      character(len=*) :: nullval
      character(len=*), dimension(nelements) :: values  
    end subroutine  FTGCVS
  end interface FTGCVS

  ! FTPCLx:

  
  interface FTPCLB
    subroutine  FTPCLB(unit,colnum,frow,felem,nelements,values,status)
      use numkind
      integer(kind=int32) :: unit,colnum,frow,felem,nelements,status
      integer(kind=int8), dimension(nelements) :: values  
    end subroutine  FTPCLB
  end interface FTPCLB
  
  interface FTPCLI
    subroutine  FTPCLI(unit,colnum,frow,felem,nelements,values,status)
      use numkind
      integer(kind=int32) :: unit,colnum,frow,felem,nelements,status
      integer(kind=int16), dimension(nelements) :: values  
    end subroutine  FTPCLI
  end interface FTPCLI
 
  interface FTPCLJ
    subroutine  FTPCLJ(unit,colnum,frow,felem,nelements,values,status)
      use numkind
      integer(kind=int32) :: unit,colnum,frow,felem,nelements,status
      integer(kind=int32), dimension(nelements) :: values  
    end subroutine  FTPCLJ
  end interface FTPCLJ
 
  interface FTPCLE
    subroutine  FTPCLE(unit,colnum,frow,felem,nelements,values,status)
      use numkind
      integer(kind=int32) :: unit,colnum,frow,felem,nelements,status
      real(kind=real32), dimension(nelements) :: values  
    end subroutine  FTPCLE
  end interface FTPCLE

  interface FTPCLD
    subroutine  FTPCLD(unit,colnum,frow,felem,nelements,values,status)
      use numkind
      integer(kind=int32) :: unit,colnum,frow,felem,nelements,status
      real(kind=real64), dimension(nelements) :: values  
    end subroutine  FTPCLD
  end interface FTPCLD




  interface FTPCLS
    subroutine  FTPCLS(unit,colnum,frow,felem,nelements,values,status)
      use numkind
      integer(kind=int32) :: unit,colnum,frow,felem,nelements,status
      character(len=*), dimension(nelements) :: values  
    end subroutine  FTPCLS
  end interface FTPCLS

  

  ! FTGKYx:

  interface FTGKYE
    subroutine  FTGKYE(unit,keyword, keyval, comment,status)
      use numkind
      integer(kind=int32)  :: unit,status
      character(len=*) :: keyword,comment
      real(kind=real32)     :: keyval
    end subroutine  FTGKYE
  end  interface FTGKYE

  interface FTGKYD
    subroutine  FTGKYD(unit,keyword, keyval, comment,status)
      use numkind
      integer(kind=int32)  :: unit,status
      character(len=*) :: keyword,comment
      real(kind=real64)     :: keyval
    end subroutine  FTGKYD
  end  interface FTGKYD

  interface FTGKYJ
    subroutine  FTGKYJ(unit,keyword, keyval, comment,status)
      use numkind
      integer(kind=int32)  :: unit,status
      character(len=*) :: keyword,comment
      integer(kind=int32)  :: keyval
    end subroutine  FTGKYJ
  end  interface FTGKYJ

  interface FTGKYL
    subroutine  FTGKYL(unit,keyword, keyval, comment,status)
      use numkind
      integer(kind=int32)  :: unit,status
      character(len=*) :: keyword,comment
      logical(kind=log32)  :: keyval
    end subroutine  FTGKYL
  end  interface FTGKYL

  interface FTGKYS
    subroutine  FTGKYS(unit,keyword, keyval, comment,status)
      use numkind
      integer(kind=int32)  :: unit,status
      character(len=*) :: keyword,comment
      character(len=*) :: keyval      
    end subroutine  FTGKYS
  end  interface FTGKYS

  ! FTPKYx:

  interface FTPKYE
    subroutine  FTPKYE(unit,keyword, keyval, decimals, comment,status)
      use numkind
      
      integer(kind=int32)  :: unit,status,decimals
      character(len=*) :: keyword,comment
      real(kind=real32)     :: keyval
    end subroutine  FTPKYE
  end  interface FTPKYE

  interface FTPKYD
    subroutine  FTPKYD(unit,keyword, keyval, decimals, comment,status)
      use numkind
      integer(kind=int32)  :: unit,status,decimals
      character(len=*) :: keyword,comment
      real(kind=real64)     :: keyval
    end subroutine  FTPKYD
  end  interface FTPKYD


  interface FTPKYJ
    subroutine  FTPKYJ(unit,keyword, keyval, comment,status)
      use numkind
      integer(kind=int32)  :: unit,status
      character(len=*) :: keyword,comment
      integer(kind=int32)  :: keyval
    end subroutine  FTPKYJ
  end  interface FTPKYJ

  interface FTPKYL
    subroutine  FTPKYL(unit,keyword, keyval, comment,status)
      use numkind
      integer(kind=int32)  :: unit,status
      character(len=*) :: keyword,comment
      logical(kind=log32)  :: keyval
    end subroutine  FTPKYL
  end  interface FTPKYL

  interface FTPKYS
    subroutine  FTPKYS(unit,keyword, keyval, comment,status)
      use numkind
      integer(kind=int32)  :: unit,status
      character(len=*) :: keyword,comment
      character(len=*) :: keyval      
    end subroutine  FTPKYS
  end  interface FTPKYS

  ! FTP2Dx:

  interface FTP2DE
    subroutine   FTP2DE(unit,group,dim1,naxis1,naxis2,image,status)
      use numkind
      integer(kind=int32)              :: unit,group,dim1,naxis1,naxis2,status
      real(kind=real32), dimension(naxis1,naxis2) :: image     
    end subroutine  FTP2DE 
  end  interface  FTP2DE  


  interface FTP2DB
    subroutine   FTP2DB(unit,group,dim1,naxis1,naxis2,image,status)
      use numkind
      integer(kind=int32)              :: unit,group,dim1,naxis1,naxis2,status
      integer(kind=int8), dimension(naxis1,naxis2) :: image     
    end subroutine  FTP2DB 
  end  interface  FTP2DB  


  interface FTP2DI
    subroutine   FTP2DI(unit,group,dim1,naxis1,naxis2,image,status)
      use numkind
      integer(kind=int32)              :: unit,group,dim1,naxis1,naxis2,status
      integer(kind=int16), dimension(naxis1,naxis2) :: image     
    end subroutine  FTP2DI 
  end  interface  FTP2DI  


  interface FTP2DJ
    subroutine   FTP2DJ(unit,group,dim1,naxis1,naxis2,image,status)
      use numkind
      integer(kind=int32)              :: unit,group,dim1,naxis1,naxis2,status
      integer(kind=int32), dimension(naxis1,naxis2) :: image     
    end subroutine  FTP2DJ 
  end  interface  FTP2DJ  

  ! FTG2Dx:

  interface FTG2DD
    subroutine   FTG2DD(unit,group,nullval,dim1,naxis1,naxis2,image,anyf,status)
      use numkind
      integer(kind=int32)              :: unit,group,dim1,naxis1,naxis2,status
      real(kind=real64)                 :: nullval
      real(kind=real64), dimension(naxis1,naxis2) :: image 
      logical(kind=log32)              :: anyf 
    end subroutine  FTG2DD 
  end  interface  FTG2DD  


  interface FTG2DE
    subroutine   FTG2DE(unit,group,nullval,dim1,naxis1,naxis2,image,anyf,status)
      use numkind
      integer(kind=int32)              :: unit,group,dim1,naxis1,naxis2,status
      real(kind=real32)                 :: nullval
      real(kind=real32), dimension(naxis1,naxis2) :: image 
      logical(kind=log32)              :: anyf 
    end subroutine  FTG2DE 
  end  interface  FTG2DE  


  interface FTG2DB
    subroutine   FTG2DB(unit,group,nullval,dim1,naxis1,naxis2,image,anyf,status)
      use numkind
      integer(kind=int32)                 :: unit,group,dim1,naxis1,naxis2,status
      integer(kind=int8)                 :: nullval
      integer(kind=int8), dimension(naxis1,naxis2) :: image 
      logical(kind=log32)                 :: anyf      
    end subroutine  FTG2DB 
  end  interface  FTG2DB  


  interface   FTG2DI
    subroutine   FTG2DI(unit,group,nullval,dim1,naxis1,naxis2,image,anyf,status)
      use numkind
      integer(kind=int32)                 :: unit,group,dim1,naxis1,naxis2,status
      integer(kind=int16)                 :: nullval      
      integer(kind=int16), dimension(naxis1,naxis2) :: image 
      logical(kind=log32)                  :: anyf      
    end subroutine  FTG2DI 
  end  interface  FTG2DI 

  interface   FTG2DJ
    subroutine   FTG2DJ(unit,group,nullval,dim1,naxis1,naxis2,image,anyf,status)
      use numkind
      integer(kind=int32),intent(in)      :: unit,group,dim1,naxis1,naxis2
      integer(kind=int32),intent(out)     :: status
      integer(kind=int32),intent(in)      :: nullval    
      integer(kind=int32), dimension(naxis1,naxis2),intent(out) :: image 
      logical(kind=log32),intent(out)        :: anyf      
    end subroutine  FTG2DJ 
  end  interface  FTG2DJ  


! FTGSVx
!******************************************************************
! dimension(:) for 'array' causes segmentation faults , removed!  
!******************************************************************


!  interface FTGSVD
!    subroutine   FTGSVD(unit,group,naxis,naxes,fpixels,lpixels,incs,nullval,array,anyf,status)
!      use numkind
!      integer(kind=int32)              :: unit,group,naxis,status
!      real(kind=real64)                :: nullval
!      real(kind=real64), dimension(:)  :: array
!      integer(kind=int32), dimension(naxis) :: naxes,fpixels,lpixels,incs
!      logical(kind=log32)              :: anyf 
!    end subroutine  FTGSVD 
!  end  interface  FTGSVD


!  interface FTGSVE
!    subroutine   FTGSVE(unit,group,naxis,naxes,fpixels,lpixels,incs,nullval,array,anyf,status)
!      use numkind
!      integer(kind=int32)              :: unit,group,naxis,status
!      real(kind=real32)                :: nullval
!      real(kind=real32), dimension(:)  :: array
!      integer(kind=int32), dimension(naxis) :: naxes,fpixels,lpixels,incs
!      logical(kind=log32)              :: anyf 
!    end subroutine  FTGSVE
!  end  interface  FTGSVE




end module CFITSIO_IFACES 
